package com.autentia.tutoriales.spring.ws.biblioteca.cliente;

import java.util.ArrayList;
import java.util.List;
import java.io.StringReader;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.w3c.dom.*;

/**
 * Spring cliente WS del servicio de consulta de libros 
 * @author Carlos García. Autentia.
 * @see http://www.mobiletest.es
 */
public class BibliotecaCliente   {
	private final String	   URI = "mailto:soporte@mobiletest.es";
	private WebServiceTemplate webServiceTemplate;

	/**
	 * Será inyectada por Spring
	 */
	public void setWebServiceTemplate(WebServiceTemplate webServiceTemplate) {
		this.webServiceTemplate = webServiceTemplate;
	}

	/**
	 * @param categoria Categoría del libro a consultar
	 * @param nivel Nivel (avanzado, medio o básico)
	 * @return Devuelve la lista de libros que cumplen los criterios especificados
	 */
	public java.util.List<Libro> getLibros(String categoria, String nivel) {
		String		 xmlRequest = this.getPeticion(categoria, nivel);
		StreamSource peticion   = new StreamSource(new StringReader(xmlRequest));
		DOMResult	 respuesta	= new DOMResult();
		List<Libro>  libros = null;
		
		boolean hayRespuesta = webServiceTemplate.sendSourceAndReceiveToResult(URI, peticion, respuesta);
		if (hayRespuesta){
			libros = this.resultToBooks((Document) respuesta.getNode());
		}
		return libros;
	}
	
	/**
	 * @return Genera la petición (payload) que se enviará al servicio Web
	 */
	private String getPeticion(String categoria, String nivel){
		StringBuffer buffer = new StringBuffer(1024);
		buffer.append("<BooksInfoRequest xmlns='http://www.adictosaltrabajo.com/spring/ws/schemas'>");
		buffer.append("<categoria>");
		buffer.append(categoria);
		buffer.append("</categoria>");
		buffer.append("<nivel>");
		buffer.append(nivel);
		buffer.append("</nivel>");
		buffer.append("</BooksInfoRequest>");
		return buffer.toString();
	}
	
	/**
	 * @return Devuelve una lista de Libro a partir del DOM
	 */
	private List<Libro> resultToBooks(Document doc){
		NodeList nodos 	 = doc.getFirstChild().getChildNodes();
		Node	 current = null;
		Libro	 libro	 = null;	
		
		ArrayList<Libro> libros = new ArrayList<Libro>();
		for (int i = 0, num = nodos.getLength(); i < num; i++){
			current = nodos.item(i);
			
			libro	= new Libro();
			libro.setEditorial(this.getProperty(current, "editorial"));
			libro.setTitulo(this.getProperty(current, "titulo"));
			libro.setPrecio(Integer.parseInt(this.getProperty(current, "precio")));
			libro.setPaginas(Integer.parseInt(this.getProperty(current, "paginas")));
			
			libros.add(libro);
		}
		
		return libros;
	}
	
	/**
	 * 
	 * @param node Nodo padre en el cual se busca la propiedad
	 * @param tag Nombre del elemento buscado
	 * @return Devuelve el valor de un elemento <padre><tag1>xxx<tag1><tagN>xxx<tagN></padre>
	 */
	private String getProperty(Node nodo, String tag){
		NodeList props  = nodo.getChildNodes();
		Node	 prop   = null;
		String	 value	= null;
		
		for (int j = 0, num = props.getLength(); j < num; j++){
			prop = props.item(j);
			if (tag.equals(prop.getNodeName())){
				value = prop.getFirstChild().getNodeValue();
				break;
			}
		}
		return value;
	}

	
}
